procedure mosfinder()
# msetscr$mosfinder.cl
# Script for generating a finder from the MKM output from makemos

# Created: 2002-Feb-08
# Last Modified: 2001-Jun-12 by J. Tufts
# Version 0.1.1

# Revision History:
# v.0.1.1 2001-Jun-12 by J. Tufts
#  - changed vector description
# v.0.1.0 2001-Feb-08 by J. Tufts
#  - First version
#  - based on makemos v.02.00 2001-Jan-30 by Keith Thompson 
#  - added multiple reference objects
#  - added slit numbering to side
#  - added LRS column positions of slitlets to side
#  - draw a faint mask
#  - added + marks on objects
#  - allow reference coords to be printed in 9 orientations
#  - changed size of NE marker and font size
#  - added tvmark color parameter
#  - if no input file is specified mfinder looks at makemos param for filename
#  - mfinder can now be run without any input and does so fairly intelligently
# To Do:
#  - Actually get XPA stuff to work

# mosfinder parameters
char	mkmfile  {"",prompt="MKM File with slit/object tables"}
char    cfgfile  {"",prompt="MOS configuration file name"}
real    disp_z1  {0,prompt="z1 for display (z1=0, z2=0 for auto)"}
real    disp_z2  {0,prompt="z2 for display (z1=0, z2=0 for auto)"}
int	color    {204,prompt="Slitlet color (202..217)"}
#bool    ds9xpa   {no,prompt="Use DS9 with XPA"}
#char    xpanm    {"ds9",prompt="XPA process name"}
bool	clobber  {no,prompt="Clobber output image file name"}

begin

#Variable definitions
char n_mkmfile,n_image     # filenames
char n_cfgfile
real n_z1, n_z2            # display values
bool auto_z, n_clobber     # Clobber exisiting output files
bool lrsimage
int  n_xbin,n_ybin,n_binning
char verstring
#Parameter variables
real a_xcen[13],a_ycen[13] # Positions of objects or slitlets from makemos
real a_xref[5],a_yref[5]   # Positions of reference objects from makemos
real a_xdref[5],a_ydref[5] # LRS distorted positions of ref. objects
real a_xdcen[13]           # LRS distorted x positions of slitlets
real n_xdcen               # x position of current slitlet
real n_xref, n_yref        # reference object coordinates
int  n_ref                 # Pointer to current ref. object
int  n_rnum                # Number of reference objects
int  n_color               # slitlet plotting color
int  n_xshift,n_yshift     # shift amounts
int  n_sltop,n_slbot
int  n_slit                # reference to currently selected slitlet
real n_xcen,n_ycen,n_xcur,n_ycur
char n_key,n_junk,n_junk2
#Top Stuff
char n_projid, n_objname, ra, dec, epoch
real n_pa, pa_mod, vec_len, x1, x2, y1, y2
char n_tele,n_inst
#Slitlet information
int  a_islr[14]            # row of interslit position
#Temporary storage files and variables
char tmpdispim,tmpshiftim,tmpcoord
char imcalstr
char d_z1,d_z2

#a_query=no
#printf("Make a finding chart (%b)? ",a_query)
#scan(a_query)
#if( a_query )  {

# Set the version string
verstring     = "v.0.1.1 2002-Jun-12"

#Startup message
printf("mfinder.cl -- %s\n\n",verstring)
printf("Finding charts are best printed with ds9:\n")
printf("  File -> Page Setup -> Fixed in Size\n")
printf("  File -> Print -> Resolution 75 dpi (don't interpolate)\n")
printf("Or you can use a screen capture program such as xv\n")
printf("-------------------------------------------------------------\n\n")

#Variable initializations
n_mkmfile = mkmfile
if (n_mkmfile == "") n_mkmfile = makemos.outname
n_cfgfile = cfgfile
n_z1 = disp_z1
n_z2 = disp_z2
n_clobber = clobber
if ((n_z1 == 0) && (n_z2 == 0)) {
  auto_z = yes
} else {
  auto_z = no
}
n_color = color
if ((n_color < 202) || (n_color > 217)) n_color = 204    #default to RED

## step 1: error checking and setup
# Determine if we are to use automatic z scaling
if( n_z1 == 0 && n_z2 == 0 )  auto_z = yes
else                                  auto_z = no
# Check to see that the input file exists and has the full extension included
if (!access(n_mkmfile)) {
  printf("ERROR: %s does not exist.\n Try using a full name with extension\n",
         n_mkmfile)
  bye
}
# read information from finder generation file
#read image file name
#read pixel positions of slitlets
# Load image header information
# BUG: no error checking use keypar to check if these exist (see modmos)
imgets (n_mkmfile//"[0]","POSANGLE"); n_pa = real(imgets.value)
#imgets (n_mkmfile//"[0]","OBJECT");   n_objname = imgets.value
imgets (n_mkmfile//"[0]","EQUINOX");  epoch = imgets.value
imgets (n_mkmfile//"[0]","CCDSUM");   print(imgets.value) | scan(n_xbin,n_ybin)
imgets (n_mkmfile//"[0]","TELESCOP"); n_tele = imgets.value
imgets (n_mkmfile//"[0]","INSTRUME"); n_inst = imgets.value
if(n_tele == "het" && n_inst == "lrs") {
  lrsimage=yes
  imgets (n_mkmfile//"[0]","RA");     ra = imgets.value
  imgets (n_mkmfile//"[0]","DEC");    dec = imgets.value
} else {
  lrsimage=no
  printf ("Image was not taken with the HET/LRS.\n")
  printf ("No astrometric distortion corrections will be applied\n")
  # Translate xy pixel coords of image center to RA and DEC
  ## ****** Possibly this should use wcstran ******
  xy2rd(n_mkmfile//"[0]",284,256,hms+,>>& "/dev/null")
  ra  = xy2rd.ra
  dec = xy2rd.dec
}
# read n_xshift,n_yshift
keypar (n_mkmfile//"[1]","XSHIFT",silent+)
if (keypar.found) n_xshift = real(keypar.value)
else n_xshift = 0.
keypar (n_mkmfile//"[1]","YSHIFT",silent+)
if (keypar.found) n_yshift = real(keypar.value)
else n_yshift = 0.
# read slitlet and object positions and information
keypar (n_mkmfile//"[1]","NAXIS2",silent+)
for (i=1;i<=int(keypar.value);i+=1) {
  tabpar (n_mkmfile//"[1]","SlitNum",i);  n_slit          = int(tabpar.value)
  tabpar (n_mkmfile//"[1]","CentX",i);    a_xcen[n_slit]  = real(tabpar.value)
  tabpar (n_mkmfile//"[1]","CentY",i);    a_ycen[n_slit]  = real(tabpar.value)
  tabpar (n_mkmfile//"[1]","DCentX",i);   a_xdcen[n_slit] = real(tabpar.value)
  if (a_xdcen[n_slit] < 0.) a_xdcen[n_slit] = a_xcen[n_slit]
}
keypar (n_mkmfile//"[2]","NAXIS2",silent+)
n_rnum = int(keypar.value)
for (i=1;i<=n_rnum;i+=1) {
  tabpar (n_mkmfile//"[2]","ObjNum",i);    n_ref          = int(tabpar.value)
  if (n_ref <= 5) {
    tabpar (n_mkmfile//"[2]","CentX",i);   a_xref[n_ref]  = real(tabpar.value)
    tabpar (n_mkmfile//"[2]","CentY",i);   a_yref[n_ref]  = real(tabpar.value)
    tabpar (n_mkmfile//"[2]","DistrtX",i); a_xdref[n_ref] = real(tabpar.value)
    tabpar (n_mkmfile//"[2]","DistrtY",i); a_ydref[n_ref] = real(tabpar.value)
  }
}
keypar (n_mkmfile//"[1]","PROJID",sil+)
if (keypar.found) n_projid = keypar.value
else n_projid = "HETxx-y-zzz"
keypar (n_mkmfile//"[1]","CFGFILE",sil+)
if (n_cfgfile == "" && keypar.found) n_cfgfile  = keypar.value
if (n_cfgfile == "" && !keypar.found) n_cfgfile = n_mkmfile//".cfg"
keypar (n_mkmfile//"[1]","OBJNAME",sil+)
if (keypar.found) n_objname = keypar.value
else n_objname = "Unknown"


## step 2: plot image
# Set mask parameters
a_islr[1]  = (1 + int(sbt[1])) / 2         #  0
for (i=2;i<14;i+=1) {
  a_islr[i] = (int(stp[i-1]) + int(sbt[i])) / 2
}
a_islr[14] = (int(stp[13]) + 512) / 2      # 512+4
# Create temporary image
tmpdispim  = mktemp("tmp$mkm-disp")
tmpshiftim = mktemp("tmp$mkm-shift")
tmpcoord   = mktemp("tmp$mkm-coord")
if(n_yshift !=0 || n_xshift !=0) {
  imshift(n_mkmfile//"[0]",tmpshiftim,n_xshift,n_yshift,interp_type="linear",
          boundary_typ="constant",constant=20000)
  n_image=tmpshiftim
} else {
  n_image=n_mkmfile//"[0]"
}
# Generate mask
imcalstr = "if (y<"//str(sbt[1])//")||(y>"//str(stp[13])//")||"
for (i=1;i<13;i+=1) {
  n_sltop = int(stp[i])
  n_slbot = int(sbt[i+1])
  imcalstr = imcalstr//"(y>"//str(n_sltop)
  imcalstr = imcalstr//"&&y<"//str(n_slbot)//")||"
}
imcalstr = imcalstr//"(x<"//str(int(slminp))//")||"
imcalstr = imcalstr//"(x>"//str(int(slmaxp))//") "
imcalstr = imcalstr//"then (0.9*im1) else im1"
imcalc(n_image,tmpdispim,imcalstr)
# BUG: if z1=0 and z2=0 it should be treated the same as makemos vals
#      that is use makemos's input or autoz if makemos's input was 0,0
if( auto_z )  {
  display(tmpdispim,1,zra+,zsc+,ycenter=0.47) | scan(d_z1,d_z2)
  n_z1=real(substr(d_z1,4,strlen(d_z1)))
  n_z2=real(substr(d_z2,4,strlen(d_z2)))
}
else  {
  display(tmpdispim, 1, zrange-, zscale-, z1=n_z1, z2=n_z2,
          ycenter=0.47)
}


## step 3: Overplot makemos information
# Slitlets
for (n_slit=1;n_slit<=13;n_slit=n_slit+1)  {
  if (a_xcen[n_slit] == -1) n_xcen = slcenp
  else if (a_xcen[n_slit] == -2) n_xcen = slbafp
  else n_xcen = a_xcen[n_slit]
  n_ycen = a_ycen[n_slit]
  # Draw slitlets
  if(access(tmpcoord)) delete(tmpcoord, ver-)
  for(i=a_islr[n_slit];i<=a_islr[n_slit + 1]; i+=1) {
    print(str(int(int(n_xcen[n_slit]) - 1))//"   "//str(i),>>tmpcoord)
    print(str(int(int(n_xcen[n_slit]) + 1))//"   "//str(i),>>tmpcoord)
  }
  tvmark(1,tmpcoord,mark="point",pointsize=1,color=n_color,label-)
  # Draw marks on slitlets
  if (n_ycen > 0.) {
    if(access(tmpcoord)) delete(tmpcoord, ver-)
    print (str(int(n_xcen+1.))//"   "//str(int(n_ycen)),>>tmpcoord)
    tvmark(1,tmpcoord,mark="plus",txsize=1,color=n_color,label-)
  }
}
# Reference objects
for (i=1;i<=n_rnum;i+=1) {
  n_xcen = a_xref[i]
  n_ycen = a_yref[i]
  if(access(tmpcoord)) delete(tmpcoord, ver-)
  print (str(int(n_xcen))//"   "//str(int(n_ycen)),>>tmpcoord)
  tvmark(1,tmpcoord,mark="circle",radii="3,6",color=n_color,label-)
}
# Slit Numbers and positions
# BUG: For finder generation these should probably come from input file
#      not msetup.cl
for (i=1;i<=13;i+=1) {
  n_xcur = slminp - (3.*6.)
  n_ycur = ((stp[i]-sbt[i]) / 2.) + 1. + sbt[i]
  printf ("%d %d 101 :text %2d\n",int(n_xcur),int(n_ycur),i) | tvmark(1,
          coords="",commands="STDIN",color=n_color,txsize=1,label-)
  n_xcur -= (5.*6.)
  n_ycur -= 9.
  if (a_xdcen[i] > 0) printf ("%d %d 101 :text %7.2f\n",int(n_xcur),
    int(n_ycur),a_xdcen[i]) | tvmark(1,coords="",commands="STDIN",
    color=n_color,txsize=1,label-)
}


## Step 4: Overplot Text
# reference object positions
n_key = "?"
while (n_key != "q") {
  n_xcen=-1
  n_ycen=-1
  if (n_key>="1" && n_key<="9") {
    n_ref = 0
    for (i=1;i<=n_rnum;i+=1) {
      if (n_xcur>(a_xref[i]-5) && n_xcur<(a_xref[i]+5) &&
          n_ycur>(a_yref[i]-5) && n_ycur<(a_yref[i]+5)) n_ref=i
    }
    if (n_ref == 0) {
      printf ("Cursor position not at a reference object, try again.\n")
    } else {
      printf ("Reference %d coordinates will be printed at position %s.\n",
              n_ref,n_key)
      n_xcur = a_xref[n_ref]
      n_ycur = a_yref[n_ref]
      n_xcen = a_xdref[n_ref]
      n_ycen = a_ydref[n_ref]
      if (n_key == "7" || n_key == "4" || n_key == "1") n_xcur -= ((16*6)+9)
      if (n_key == "9" || n_key == "6" || n_key == "3") n_xcur +=  4
      if (n_key == "7" || n_key == "8" || n_key == "9") n_ycur +=  9
      if (n_key == "1" || n_key == "2" || n_key == "3") n_ycur -= 15
      if (n_key == "5" || n_key == "8" || n_key == "2") n_xcur -= ((9*6)-4)
      if (n_key == "5" || n_key == "4" || n_key == "6") n_ycur -= 3
      if(access(tmpcoord)) delete(tmpcoord,ver-)
      #print (str(int(n_xcur))//"   "//str(int(n_ycur)),>>tmpcoord)
      #tvmark(1,tmpcoord,mark="circle",radii="3,6",color=n_color,label-)
      printf ("%d %d 101 :text (%0.2f,%0.2f)\n",int(n_xcur),int(n_ycur),
              n_xcen,n_ycen) | tvmark(1,coords="",commands="STDIN",
              color=n_color,txsize=1,label-)
    }
  } else if (n_key == "?") {
    printf ("\nPosition cursor on reference and choose from the following:\n")
    printf (" Text Location:\n\n")
    printf ("   7   8    9\n")
    printf ("   4 Object 6\n")
    printf ("   1   2    3\n\n")
    printf (" ? - Reprint this menu\n")
    printf (" q - Quit\n")
  } else {    
    printf("Try again, get more sleep, and buy better gin.\n")
  }
  n_junk2=fscan(imcur,n_xcur,n_ycur,n_junk,n_key)
}


## Step 5: Print Top Crap
printf("0 555 101 :text %s\n", n_projid) | tvmark(1,coords="", 
       commands="STDIN", color=n_color, txsize=2)
printf("0 535 101 :text %s\n", n_objname) | tvmark(1,coords="", 
       commands="STDIN", color=n_color, txsize=2)
printf("140 555 101 :text %s %s (%s)\n", ra,dec,epoch) | tvmark(1,
       coords="",commands="STDIN", color=n_color, txsize=2)
printf("290 535 101 :text PA=%0.2f\n", n_pa) | tvmark(1,coords="", 
       commands="STDIN", color=n_color, txsize=2)
printf("0 515 101 :text %s\n",n_cfgfile) | tvmark(1,coords="", 
       commands="STDIN", color=n_color, txsize=2)

## step 4:  N-E vectors 
## REM these are PA=0,90 vectors on a PA!=0 ROTATED coord system --- 
## not PA!=0 vector on a PA=0 coordinate system
## ask for position
vec_len = 50.0
pa_mod = (n_pa + 360.0)%360.0
printf("\nPosition cursor at desired location of N-E vectors\n"
printf(" q - Draw N-E vector at cursor position\n")
printf("They will be %.0f pix long, and will look approx. like:\n\n",vec_len)
if( pa_mod <= 23.0 )        printf("      N\n      |\n  E---'\n")
else if( pa_mod <= 67.0 )   printf ("  E  N\n   \\/\n")
else if( pa_mod <= 113.0 )  printf ("  E\n  |\n  `---N\n")
else if( pa_mod <= 157.0 )  printf ("   E\n  <\n   N\n")
else if( pa_mod <= 203.0 )  printf ("  ,---E\n  |\n  N\n")
else if( pa_mod <= 247.0 )  printf ("   /\\\n  N  E\n")
else if( pa_mod <= 293.0 )  printf ("  N---.\n      |\n      E\n")
else if( pa_mod <= 337.0 )  printf ("  N\n   >\n  E\n")       
else                        printf ("      N\n      |\n  E---'\n")
n_junk2=fscan(imcur,n_xcur,n_ycur,n_junk,n_key)
x1 = n_xcur
y1 = n_ycur
## calculate vector ends
## PA=0 => N is just dx=0,dy=1; PA=90=> N dx=1,dy=0
##     so dx=sin(n_pa)    dy=cos(n_pa)
x2 = x1 + vec_len*sin(n_pa*pi/180.0)
y2 = y1 + vec_len*cos(n_pa*pi/180.0)
printf("%.1f %.1f 101 s\n%.1f %.1f 101 s\n", x1, y1, x2, y2) | tvmark(1,
       coords="", commands="STDIN", color=n_color)
## now for "N"
if( 180.0 < pa_mod && pa_mod < 270.0)   {    ### have to shift left a bit
  printf("%.1f %.1f 101 :text N\n", (x2-16), y2) | tvmark(1,
         commands="STDIN", coords="", color=n_color, txsize=1)
}
else  {
  printf("%.1f %.1f 101 :text N\n", x2, y2) | tvmark(1,
         commands="STDIN", coords="", color=n_color, txsize=1)
}
## E vector: PA=0 => dx=-1,dy=0;  PA=90=> dx=0,dy=1
##    dx=-cos(n_pa), dy=sin(n_pa)
x2 = x1 - vec_len*cos(n_pa*pi/180.0)
y2 = y1 + vec_len*sin(n_pa*pi/180.0)
#debug
#printf("%f %f    %f %f\n", x1, y1, x2, y2)
printf("%.1f %.1f 101 s\n%.1f %.1f 101 s\n", x1, y1, x2, y2) | tvmark(1,
       coords="", commands="STDIN", color=n_color)
## "E"
if( pa_mod > 270.0 )  {
  printf("%.1f %.1f 101 :text E\n", (x2-16), y2) | tvmark(1,
         commands="STDIN", coords="", color=n_color, txsize=1)
}
else  {
  printf("%.1f %.1f 101 :text E\n", x2, y2) | tvmark(1,
         commands="STDIN", coords="", color=n_color, txsize=1)
}


## step 5: 60'' bar    y=520; 0.47''/pix=> 128 pix
printf("300 525 101 s\n300 520 101 s\n428 520 101 s\n428 525 101 s\n") |
       tvmark(1,coords="", commands="STDIN", color=n_color, txsize=1)
printf("435 520 101 :text 60 arcsec\n") | tvmark(1, coords="", 
       commands="STDIN", color=n_color, txsize=1)

## Delete temporary files
if(access(tmpshiftim//".fits")) imdelete(tmpshiftim, verify-)
if(access(tmpdispim//".fits")) imdelete(tmpdispim, verify-)
if(access(tmpshiftim//".imh")) imdelete(tmpshiftim, verify-)
if(access(tmpdispim//".imh")) imdelete(tmpdispim, verify-)
if(access(tmpcoord)) delete(tmpcoord,ver-)


end
